#include <libce.h>
#include <apic.h>

// registri tastiera
const ioaddr iSTR = 0x64;
const ioaddr iRBR = 0x60;
const ioaddr iCMD = 0x64;
const ioaddr iTBR = 0x60;

// registri timer 0
const ioaddr iCWR = 0x43;
const ioaddr iCTR0_LOW = 0x40;
const ioaddr iCTR0_HIG = 0x40;

extern volatile natw* video;
extern natb attr;

void abilita_intr_kbd()
{
	outputb(0x60, iCMD);
	outputb(0x61, iTBR);
}

void disabilita_intr_bkd()
{
	outputb(0x60, iCMD);
	outputb(0x61, iTBR);
}

void avvia_timer()
{
	outputb(0x36, iCWR);
	outputb((natb)50000, iCTR0_LOW);
	outputb((natb)(50000 >> 8), iCTR0_HIG);
}

bool volatile fine = false;

extern "C" void a_tastiera();
extern "C" void c_tastiera()
{
	natb c;
	inputb(iRBR, c);
	if (c == 0x01)	// make code di ESC
		fine = true;
	else {
		for (int i = 0; i < 8; i++) {
			char_write('0' + !!(c & 0x80));
			for (volatile long j = 0; j < 50000000; j++)
				;
			c <<= 1;
		}
		char_write('\n');
	}
	apic_send_EOI();
}

char spinner[] = { '|', '/', '-', '\\' };
const int KBD_TYPE = 0x40;
const int CTR_TYPE = 0x45;

extern "C" void a_timer();
extern "C" void c_timer()
{
	static unsigned timerpos = 0;
	static unsigned ctr = 0;

	//if (ctr++ % 64) {
	//	apic_send_EOI();
	//	return;
	//}

	video[12*80+60] = (attr << 8) | spinner[timerpos % 4];
	timerpos++;
	apic_send_EOI();
}

int main()
{
	unsigned int spinpos = 0;

	clear_screen(0x0f);

	// tastiera
	apic_set_VECT(1, KBD_TYPE);
	trap_init(KBD_TYPE, a_tastiera);
	apic_set_MIRQ(1, false);
	abilita_intr_kbd();

	// timer
	apic_set_VECT(2, CTR_TYPE);
	trap_init(CTR_TYPE, a_timer);
	apic_set_MIRQ(2, false);
	apic_set_TRGM(2, false);
	avvia_timer();

	while (!fine) {
		video[12*80+40] = (attr << 8) | spinner[spinpos % 4];
		spinpos++;
	}
	return 0;
}
